/* $Id: convgeocloc.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "form.h"
#include "convgeocloc.h"

using namespace GenLib2;

namespace GCL
{

// default constructor

ConvGeocLoc::ConvGeocLoc() : Ellipsoid(), _rot_matrix()
{
	_bb = _lb = _hb = 0.;
	_xb = _yb = _zb = 0.;
}


// set reference ellipsoid parameters

void ConvGeocLoc::set_reference_point(double bb, double lb, double hb)
{
	_bb = bb;
	_lb = lb;
	_hb = hb;

	geo_to_cart(_bb, _lb, _hb, _xb, _yb, _zb);

	double sp = sin(_bb);
	double cp = cos(_bb);
	double sl = sin(_lb);
	double cl = cos(_lb);
	
	_rot_matrix(0,0) = -sl;
	_rot_matrix(0,1) =  cl;
	_rot_matrix(0,2) =  0.;

	_rot_matrix(1,0) = -cl * sp;
	_rot_matrix(1,1) = -sl * sp;
	_rot_matrix(1,2) =  cp;

	_rot_matrix(2,0) =  cl * cp;
	_rot_matrix(2,1) =  sl * cp;
	_rot_matrix(2,2) =  sp;
}


// get reference point

void ConvGeocLoc::get_reference_point (double& bb, double& lb,
double& hb) const
{
	bb = _bb;
	lb = _lb;
	hb = _hb;
}


// print

void ConvGeocLoc::print(ostream& os) const
{
	Ellipsoid::print(os);
	os << endl;

	Form fix(8); fix.fixed().width(12);
	os << "Reference point" << endl;
	os << "   latitude  = " << fix(_bb * rho_deg) << " deg\n";
	os << "   longitude = " << fix(_lb * rho_deg) << " deg\n";

	fix.precision(3);
	os << "   height    = " << fix(_hb) << " m\n";
	os << "   X         = " << fix(_xb) << " m\n";
	os << "   Y         = " << fix(_yb) << " m\n";
	os << "   Z         = " << fix(_zb) << " m\n";
	//os << _rot_matrix << endl;
}


// conversion from geodetic to local coordinates

void ConvGeocLoc::geo_to_loc(double b, double l, double h,
double& u, double& v, double& w) const
{
	double x = 0., y = 0., z = 0.;

	geo_to_cart(b, l, h, x, y, z);
	//cout << x << " " << y << " " << z << endl;

	double dx = x - _xb;
	double dy = y - _yb;
	double dz = z - _zb;

	u = _rot_matrix(0,0) * dx + _rot_matrix(0,1) * dy + _rot_matrix(0,2) * dz;
	v = _rot_matrix(1,0) * dx + _rot_matrix(1,1) * dy + _rot_matrix(1,2) * dz;
	w = _rot_matrix(2,0) * dx + _rot_matrix(2,1) * dy + _rot_matrix(2,2) * dz;
}


// conversion from local to geodetic coordinates

void ConvGeocLoc::loc_to_geo(double u, double v, double w,
double& b, double& l, double& h) const
{
	double x = _rot_matrix(0,0) * u + _rot_matrix(1,0) * v
		+ _rot_matrix(2,0) * w + _xb;
	double y = _rot_matrix(0,1) * u + _rot_matrix(1,1) * v
		+ _rot_matrix(2,1) * w + _yb;
	double z = _rot_matrix(0,2) * u + _rot_matrix(1,2) * v
		+ _rot_matrix(2,2) * w + _zb;

	cart_to_geo(x, y, z, b, l, h);
}

} // namespace GCL
