/* $Id: convgeocloc.h 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GCL_CONVGEOCLOC_INCLUDED
#define GCL_CONVGEOCLOC_INCLUDED

#include "ellipsoid.h"
#include "drotmatrix3.h"

using namespace GenLib2;

namespace GCL
{

/**@brief Conversion between geocentric and local coordinates class
 *
 * This class provide the transformation of cartesian coordinates
 * between the geocentric and a local topocentric location.
 * The topocentric coordinate system is also referred as
 * local reference system (LRS).
 *
 * The origin of the local coordinate system is in
 * reference point B.
 * The reference point B is defined in respect to the reference ellipsoid
 * by the geodetic coordinates bb, lb and hb.
 * - The w-axis coincides with the ellipsoid vertical and points towards
 *   the zenit.
 * - The v-axis is in the meridian plane and perpendicular
 *   to the w-axis.
 * - The u-axis is perpendicular to the vw-plane
 *   and completing the right-handed system.
 *
 * Attention, in geodesy and surveying the local coordinate system is defined
 * as a left-handed system!
 *
 * References:
 *  - Karl Kraus, Photogrammetrie, Band 2, Abschnitt B 5.3.1.
 *  - Heck: Rechenverfahren und Auswertemodelle der Landesvermessung,
 *    Abschnitt 2.3 und 2.6.
 *
 * @author Dirk Stallmann
 */
class ConvGeocLoc : public Ellipsoid
{
public:
	/**@name constructor/destructors
	 */
	//@{
		/// default constructor
		ConvGeocLoc();

		/// destructor
		virtual ~ConvGeocLoc() {}
	//@}

	/**@name set/get reference point
	 * Set or get the definition of the reference point B
	 * in geodetic coordinates.
	 */
	//@{
		/**@brief set reference point
		 *
		 * Geodetic coordinates of the reference point:
		 * @param[in] bb  geographic latitude
		 * @param[in] lb  geographic longitude
		 * @param[in] hb  ellipsoid height
		 */
		void set_reference_point(double bb, double lb,
			double hb);

		/**@brief get reference point
		 *
		 * Geodetic coordinates of the reference point:
		 * @param[out] bb  geographic latitude
		 * @param[out] lb  geographic longitude
		 * @param[out] hb  ellipsoid height
		 */
		void get_reference_point(double& bb, double& lb,
			double& hb) const;
	//@}

	/**@brief print transformation parameters
	 *  @param[out] os   output stream
	 */
	void print(ostream& os) const;

	/**@name geodetic / local topocentric coordinates conversion
	 * Conversion between geodetic (b, l, h) and
	 * local topocentric (u, v, w) coordinates.
	 */
	//@{
		/**@brief conversion from geodetic to local coordinates
		 * @param[in]  b,l    geographic latitude and longitude
		 * @param[in]  h      ellipsoid height
		 * @param[out] u,v,w  local coordinates
		 */
		void geo_to_loc(double b, double l, double h,
			double& u, double& v, double& w) const;

		/**@brief conversion from local to geodetic coordinates
		 * @param[in]  u,v,w  local coordinates
		 * @param[out] b,l    geographic latitude and longitude
		 * @param[out] h      ellipsoid height
		 */
		void loc_to_geo(double u, double v, double w,
			double& b, double& l, double& h) const;
	//@}

protected:
	/// geographic latitude of reference point B
	double	 	_bb;

	/// geographic longitude of reference point B
	double	 	_lb;

	///  ellipsoid height of reference point B
	double	 	_hb;

	///  X-coordinate of reference point B
	double	 	_xb;

	///  Y-coordinate of reference point B
	double	 	_yb;

	///  Z-coordinate of reference point B 
	double	 	_zb;

	/// rotation matrix
	DRotMatrix3 	_rot_matrix;
};

} // namespace GCL

#endif
