/* $Id: ellipsoid.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <cstring>
#include <cassert>
#include <stdio.h>
#include <fstream>
#include "form.h"
#include "ellipsoid.h"

using namespace GenLib2;


namespace GCL
{

// default constructor

Ellipsoid::Ellipsoid() : _name()
{
	set("unit_sphere", 1., 0.);
}


// set reference ellipsoid parameters

void Ellipsoid::set(const char* name, double a, double f)
{
	_name = name;
	_a = a;
	_f = f;

	calc_params();
	calc_coeffs();
}


// constructor with name

Ellipsoid::Ellipsoid(const string& name) : _name(name)
{
	if (set("ell_pars.db", name) != true)
		cerr << "\nEllipsoid::Ellipsoid: Unknown ellipsoid: "
			<< name << endl;
}


// set reference ellipsoid by name using input stream

bool Ellipsoid::set(istream& is, const string& name)
{
	const int max = 256;
	char buf[max];
	char str[max];
	double a = 0., rf = 0.;

	char c = 0;
	while (1)
		{
		is >> c;
		if (!is) return false; // reach EOF

		// eat comments
		if (c == '#')
			while (is.get(c) && c != '\n');
		else
			{
			is.putback(c);
			is.get(buf, max);
			sscanf(buf, "%s", str); // old c-style
			if (strcmp(str, name.c_str()) == 0)
				{
				//sscanf(buf, "%*s%lf%lf", &_a, &_e2); // old c-style
				sscanf(buf, "%*s%lf%lf", &a, &rf); // old c-style
				set(name.c_str(), a, 1. / rf);
				
				return true;
				}
			}
		}
	
	return false;
}


// set reference ellipsoid by name using stream

bool Ellipsoid::set(const string& fname, const string& name)
{
	ifstream is(fname.c_str());
	if (!is)
		{
		cerr << "\nEllipsoid::set: Cannot open input file: "
			<< fname << endl;
		return false;
		}
	
	return set(is, name);
}


// set reference ellipsoid by name using default database

bool Ellipsoid::set(const string& name)
{
	return set("ell_pars.db", name);
}


// print

void Ellipsoid::print(ostream& os) const
{
	os << "Ellipsoid name            :  " << _name << "\n";
	Form fix(3); fix.fixed().width(12);
	os << "Semimajor axis            :  a    = " << fix(_a) << " m\n";
	os << "Semiminor axis            :  b    = " << fix(_b) << " m\n";
	os << "Polar radius of curvature :  c    = " << fix(_c) << " m\n";
	fix.precision(8);
	os << "Inverse flattening        :  1/f  = " << fix(1. / _f) << endl;
	fix.precision(10);
	os << "First eccentricity        :  e^2  = " << fix(_e2) << endl;
	os << "Second eccentricity       :  e'^2 = " << fix(_e22) << endl;
}


// Meridianbogenlaenge (in m) der ellipsoidischen Breite (in rad)

double Ellipsoid::arc(double b) const
{
	double s_p = sin(b);
	double s2_p = s_p * s_p;
	double s4_p = s2_p * s2_p;
	double s6_p = s2_p * s4_p;
	double s8_p = s4_p * s4_p;

	double aux = 1. + _a2 * s2_p + _a4 * s4_p + _a6 * s6_p + _a8 * s8_p;
	return (_c * (_a0 * b - _a1 * s_p * cos(b) * aux));
}


// Ellipsoidischen Breite (in rad) der Meridianbogenlaenge (in m)

double Ellipsoid::inv_arc(double g) const
{
	const int imax = 10;
	const double xacc = 1e-10;
	const double h = 1e-9;
	double fn = 0., df = 0.;
	double dx = 0.;

	double b = g / _a; // initial guess
	for (int i = 0; i < imax; i++)
		{
		funcd(b, h, fn, df);
		//cout << "phi=" << phi << endl;
		//cout << "dx=" << dx*MPC::rho_deg << endl;
		dx = (g - fn) / df;
		b += dx;
		if (fabs(dx) < xacc) return b; // convergence
		}

	cerr << "\nEllipsoid::inv_arc: Maximum number of iterations exceeded.\n";
	return 0.;
}


// isometric latitude from ellipsoidic latitude

double Ellipsoid::q(double b) const
{
	double e = sqrt(_e2);
	double esb = e * sin(b);
	return log(tan(M_PI_4 + b / 2.) * pow((1. - esb) / (1. + esb), e / 2.));
}


// ellipsoidic latitude from isometric latitude

double Ellipsoid::inv_q(double q) const
{
	double e = sqrt(_e2);
	double b0 = 0.;
	double b = 2. * atan(exp(q)) - M_PI_2;

	do
		{
		b0 = b;
		double esb = e * sin(b);
		b = 2. * atan(exp(q) * pow((1. - esb) / (1. + esb), -e / 2.)) - M_PI_2;
		//cout << b0 << " " << b << " " << b - b0 << endl;
		} while (fabs(b - b0) > 1e-10);
	
	return b;
}


// geocentric from geographic latitude

double Ellipsoid::geog_to_geoc(double b) const
{
	return atan((1. - _e2) * tan(b));
}


// geographic from geocentric latitude

double Ellipsoid::geoc_to_geog(double psi) const
{
	return atan(tan(psi) / (1. - _e2));
}


// conversion from geodetic to cartesian coordinates

void Ellipsoid::geo_to_cart(double b, double l, double h,
double& x, double& y, double& z) const
{
	double sin_b = sin(b);
	double N = _a / sqrt(1. - _e2 * sin_b * sin_b);
	double k = (N + h) * cos(b);

	x = k * cos(l);
	y = k * sin(l);
	z = ((1. - _e2) * N + h) * sin_b;

	return;
}


// conversion from cartesian to geodetic coordinates
// using iterative solution

void Ellipsoid::cart_to_geo(double x, double y, double z,
double& b, double& l, double& h) const
{
	const double eps_h = 1e-3;
	const double eps_b = 1e-10;
	const int imax = 30;

	double q = sqrt(x * x + y * y);
	l = atan2(y, x);
	h = 0.;
	double N = _a;
	for (int i = 0; i < imax; i++)
		{
		double h0 = h;
		double b0 = b;
		b = atan(z / q / (1 - _e2 * N / (N + h)));
		double sin_b = sin(b);
		N = _a / sqrt(1. - _e2 * sin_b * sin_b);
		h = q / cos(b) - N;
		//cout << i << ": "<< b << ", " << h << endl;
		if (fabs(h - h0) < eps_h && fabs(b - b0) < eps_b) return;
		}

	cerr << "\nEllipsoid::cart_to_geo: "
		"Maximum number of iterations exceeded.\n";
	return;
}


// calculate parameters
// derive some usefull ellipsoid parameters from a and f

void Ellipsoid::calc_params()
{
	//_e2 = _f + _f - _f * _f;
	_e2 = _f * (2. - _f);
	_b = _a * sqrt(1. - _e2);
	_c = _a * _a / _b;
	//_f = (_a - _b) / _a;
 	_e22 = (_a * _a) / (_b * _b) - 1.;
}


// calculate coefficients

void Ellipsoid::calc_coeffs()
{
 	double e2 = (_a * _a) / (_b * _b) - 1.; // second eccentricity squared
	double e4 = e2 * e2;
	double e6 = e2 * e4;
	double e8 = e4 * e4;

	_a0 = 1. - 3. * e2 * (1. - 15. * e2 * (1. - 35. * e2 *
		(1. - 63. * e2 * (1. - 99. * e2 / 100.) / 64.) / 36.) / 16.) / 4.;
	_a1 = 3. * e2 * (1. - 25. * e2 * (1. - 77. * e2 * (1. - 837. * e2 *
		(1.- 2123. * e2 / 1860.) / 704.) / 60.)/ 16.) / 4.;
	_a2 = 5. * e2 *(1. - 139. * e2 *(1. - 1087. * e2 *
		(1. - 513427. * e2 / 521760.) / 1112.) / 144.) / 8.;
	_a4 = 35. * e4 * (1. - 125.* e2 *
		(1. - 221069. * e2 / 150000.) / 64.) / 72.;
	_a6 = 105. * e6 * (1. - 1179. * e2 / 400.) / 256.;
	_a8 = 231. * e8 / 640.;
}


// supply function value fn and derivative
// see Numerical Recipies in C, 2nd Ed., p. 186ff

void Ellipsoid::funcd(double x, double h, double& fn, double& df) const
{
	fn = arc(x);				// function
	df = (arc(x + h) - fn) / h;	// derivative
}


} // namespace GCL
