/* $Id: ellipsoid.h 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GCL_ELLIPSOID_INCLUDED
#define GCL_ELLIPSOID_INCLUDED

#include <string>
#include <iostream>
#include <cmath>
#include "mpc.h"

using namespace std;
using namespace GenLib2;

/// Geodetic Computation Library (GCL)
namespace GCL
{

/**@brief Reference ellipsoid class
 *
 * This class provide geometric parameters and coordinate transformations
 * in the ellipsoidal reference system.
 * 
 * A reference ellipsoid is a mathematically-defined surface that approximates
 * the geoid, the true figure of the Earth or other planetary body (from Wikipedia).
 * The system is defined by a rotational ellipsoid with flattened poles.
 * The center of the reference ellipsoid coincidence with the earth center.
 * The ellipsoid is created by rotating the meridian ellipse about its
 * minor axis.
 * Reference ellipsoids are defined by two parameters, e.g.
 * the semi-major axis (equatorial radius) a and the flattening f.
 * Sometimes other combinations parameters are used, e.g.
 * the semi-minor axis b or the first eccentricity e.
 *
 * Geocentric coordinates (earth-centered) define the position of a
 * 3D point with respect to the origin of the ellipsoid.
 * Using cartesian (x, y, z) coordinates the origin is at the center of
 * the ellipsoid:
 * - The z-axis is directed to the northern ellipsoidal pole (along the minor
 * axis) and
 * - the x-axis is directed to the ellipsoidal zero meridian.
 * - The y-axis completing the right-handed system. 
 * 
 * (Ellipsoidal) geographic or geodetic coordinates define the position of a
 * 3D point with respect on the surface of the ellipsoid with:
 * - the ellipsoidal latitude b (= Breite),
 * - the ellipsoidal longitude l (= Laenge) and
 * - the ellipsoidal height h.
 * 
 * The ellipsoid (or geodetic) height
 * is the distance from the reference ellipsoid to the point in
 * the direction normal to the ellipsoid.
 *
 * References:
 *  - Heck: Rechenverfahren und Auswertemodelle der Landesvermessung,
 *    Wichmann Verlag, Karlsruhe, 1987.
 *  - Grossmann: Geodaetische Rechnungen und Abbildungen
 *    in der Ladesvermessung, 3rd Ed., Wittwer, Stuttgart, 1976.
 *  - Torge: Geodesy, de Gruyter, Berlin, New York, 1980.
 *  - Numerical Recipies in C, 2nd Ed., p. 186ff, 365ff.
 *
 * @author Dirk Stallmann
 */
class Ellipsoid
{
public:
	/**@name constructor/destructors
	 */
	//@{
		/**@brief default constructor
		 *
		 *  Initilize reference ellipsoid with unit sphere.
		 */
		Ellipsoid();

		/**@brief constructor with ellipsoid parameters
		 *
		 * @param[in] name   name of the reference ellipsoid
		 * @param[in] a      semimajor axis
		 * @param[in] f      flattening
		 */
		Ellipsoid(const char* name, double a, double f)
		{
			set(name, a, f);
		}
	
		/**@brief constructor with ellipsoid name
		 *
		 * Set the parameters of the reference ellipsoid using
		 * the data base.
		 * The default file name of the data base is ell_pars.db.
		 * @param[in] name   name of the reference ellipsoid in the data base
		 */
		Ellipsoid(const string& name);

		/// destructor
		virtual ~Ellipsoid() {}
	//@}

	/**@name set reference ellipsoid
	 * Set the parameters of the reference ellipsoid directly
	 * or by using the data base.
	 * The default file name of the data base is ell_pars.db.
	 */
	//@{
		/**@brief set reference ellipsoid parameters directly
		 *
		 * @param[in] name   name of the reference ellipsoid
		 * @param[in] a      semimajor axis
		 * @param[in] f      flattening
		 */
		void set(const char* name, double a, double f);

		/**@brief using data base read from stream
		 *
		 * @param[in] is     input stream
		 * @param[in] name   name of the reference ellipsoid
		 * @return true = no problem, false = problem occurred
		 */
		bool set(istream& is, const string& name);

		/**@brief using data base in file
		 *
		 * @param[in] fname   file name
		 * @param[in] name    name of the reference ellipsoid
		 * @return true = no problem, false = problem occurred
		 */
		bool set(const string& fname, const string& name);

		/**@brief using local default file
		 *
		 * @param[in] name    name of the reference ellipsoid
		 * @return true = no problem, false = problem occurred
		 */
		bool set(const string& name);
	//@}

	/**@brief print ellipsoid parameters
	 * @param[in] os   output stream
	 */
	void print(ostream& os) const;

	/// get ellipsoid name
	const char* ell_name() const { return _name.c_str(); }

	/// check if valid reference ellipsoid
	bool is_valid() const { return (_a > 0.) ? true : false; }

	/**@brief radius of curvature in the prime vertical
	 *
	 * see Torge, (3.26), S. 51
	 * @param[in] b   geographic latitude
	 */
	double nradius(double b) const;

	/**@brief meridian radius of curvature
	 *
	 * see Torge, (3.24), S. 50
	 * @param[in] b   geographic latitude
	 */
	double mradius(double b) const;
	
	/**@brief geocentric distance
	 *
	 * @param[in] psi  geocentric latitude
	 */
	double dist(double psi) const;

	/**@brief length of meridian arc
	 *
	 * @param[in] b   geographic latitude
	 */
	double arc(double b) const;
	
	/**@brief latitude of meridian arc
	 *
	 * @param[in] g   length of meridian arc
	 */
	double inv_arc(double g) const;

	/**@brief isometric latitude from ellipsoidic latitude
	 *
	 * see Heck, (6.92b), S. 202
	 * @param[in] b   geographic latitude
	 */
	double q(double b) const;

	/**@brief ellipsoidic latitude from isometric latitude
	 *
	 * see Numerical Recipies in C, 2nd Ed., p. 365ff
	 * @param[in] q   isometric latitude
	 */
	double inv_q(double q) const;

	/**@name conversion between geographic and geocentric latitude
	 * see Grossmann, 13.4, S. 9 and personal notes
	 */
	//@{
		/**@brief geocentric from geographic latitude
		 * @param[in] b    geographic latitude
		 * @return geocentric latitude
		 */
		double geog_to_geoc(double b) const;

		/**@brief geographic from geocentric latitude
		 * @param[in] psi  geocentric latitude
		 * @return geographic latitude
		 */
		double geoc_to_geog(double psi) const;
	//@}

	/**@name geocentric cartesian / geodetic coordinates conversion
	 * Conversion between geocentric cartesian (x, y, z) and
	 * geodetic (b, l, h) coordinates.
	 */
	//@{
		/**@brief conversion from geodetic to cartesian coordinates
		 *
		 * @param[in]  b,l     geographic latitude and longitude
		 * @param[in]  h       ellipsoid height
		 * @param[out] x,y,z   geocentric cartesian coordinates 
		 */
		void geo_to_cart(double b, double l, double h,
			double& x, double& y, double& z) const;

		/**@brief conversion from cartesian to geodetic coordinates
		 *
		 * @param[in]  x,y,z   geocentric cartesian coordinates 
		 * @param[out] b,l     geographic latitude and longitude
		 * @param[out] h       ellipsoid height
		 */
		void cart_to_geo(double x, double y, double z,
			double& b, double& l, double& h) const;
	//@}

protected:
	/// name of rotational ellipsoid
	string		_name;

	/// semimajor axis
	double	 	_a;	

	/// semiminor axis
	double	 	_b;	

	/// polar radius of curvature
	double	 	_c;	

	/// flattening
	double	 	_f;	

	/// square of first eccentricity
	double	 	_e2;

	/// square of second eccentricity
	double	 	_e22;

private:
	/// coeffs. for computation of the meridian arc
	//@{
		/// coefficent a0
		double		_a0;

		/// coefficent a1
		double		_a1;

		/// coefficent a2
		double		_a2;

		/// coefficent a4
		double		_a4;

		/// coefficent a6
		double		_a6;

		/// coefficent a8
		double		_a8;
	//@}

	/**@brief calculate ellipsoid parameters
	 *
	 * see Heck, (3.5)+(3.6), S.63
	 * and personal notes
	 */
	void calc_params();

	/// calculate coefficients for computation of the meridian arc
	void calc_coeffs();

	/// get length of the meridian arc and first derivative 
	void funcd(double x, double h, double& fn, double& df) const;
};


// implementations ...

// radius of curvature in the prime vertical

inline double Ellipsoid::nradius(double phi) const
{
	double sin_phi = sin(phi);
	return (_a / sqrt(1. - _e2 * sin_phi * sin_phi));
}


// meridian radius of curvature

inline double Ellipsoid::mradius(double phi) const
{
	double sin_phi = sin(phi);
	return (_a * (1. - _e2) / pow(1. - _e2 * sin_phi * sin_phi, 1.5));
}


// geocentric distance

inline double Ellipsoid::dist(double psi) const
{
	double cos_psi = cos(psi);
	double b2 = _a * _a * (1. - _e2);
	return sqrt(b2 / (1. - _e2 * cos_psi * cos_psi));
	//return _a * sqrt((1. - _e2) / (1. - _e2 * cos_p * cos_p));
}

} // namespace GCL

#endif
