/* $Id: gclio.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#include <cassert>
#include <sstream>
#include "gclio.h"
#include "form.h"

namespace GCL
{

// keyword map

GCLIO::map_t GCLIO::_map[] =
{
	{ UNKNOWN,			"unknown" },
	{ VERSION,			"version" },
	{ ELLIPSOID,		"ellipsoid" },
	{ MAP_PROJECTION,	"map_projection" },
	{ UNIT_ANGLE,		"unit_angle" },
	{ ORIGIN_LATITUDE,		"origin_latitude" },
	{ ORIGIN_LONGITUDE,		"origin_longitude" },
	{ SCALE,			"scale" },
	{ OFFSET_X,			"offset_x" },
	{ OFFSET_Y,			"offset_y" },
	{ STD_PARALLEL_1,	"std_parallel_1" },
	{ STD_PARALLEL_2,	"std_parallel_2" },
	{ REFERENCE_POINT,	"reference_point" },
	{ START_DATA,		"start_data" },
	{ END_DATA,			"end_data" },
	{ END_OF_MAP,		0 }
};


// get code

GCLIO::ecode GCLIO::code(const char* name)
{
	for (int i = 0; _map[i].name != 0; i++)
		{
		if (strcmp(name, _map[i].name) == 0)
			return (ecode)_map[i].code;
		}

	return (ecode)_map[0].code;
}


// read header

bool GCLIO::read_header(istream& is)
{
	string line;
	string str;
	UnitAngle uangle;
	double rho = 1.;
	bool hdr = true;

	while (hdr)
		{
		getline(is, line);
		//cout << line << endl;
		string keyword;
		istringstream iss(line);
		iss >> keyword;
		//cout << "keyword: " << keyword << endl;

		ecode key = code(keyword.c_str());
		//cout << key << endl;

		switch (key)
			{
			case VERSION:
				iss >> _major >> _minor >> _patch;
				_eflags |= VERSION;
				break;

			case ELLIPSOID:
				iss >> _ell_name;
				_eflags |= ELLIPSOID;
				break;

			case MAP_PROJECTION:
				iss >> _map_name;
				_eflags |= MAP_PROJECTION;
				break;

			case UNIT_ANGLE:
				iss >> str;
				_acode = uangle.code(str.c_str());
				rho = uangle.rho(_acode);
				_eflags |= UNIT_ANGLE;
				break;

			case ORIGIN_LATITUDE:
				iss >> _b0;
				_b0 /= rho;
				_eflags |= ORIGIN_LATITUDE;
				break;

			case ORIGIN_LONGITUDE:
				iss >> _l0;
				_l0 /= rho;
				_eflags |= ORIGIN_LONGITUDE;
				break;

			case SCALE:
				iss >> _m0;
				_eflags |= SCALE;
				break;

			case OFFSET_X:
				iss >> _x0;
				_eflags |= OFFSET_X;
				break;

			case OFFSET_Y:
				iss >> _y0;
				_eflags |= OFFSET_Y;
				break;

			case STD_PARALLEL_1:
				iss >> _b1;
				_b1 /= rho;
				_eflags |= STD_PARALLEL_1;
				break;

			case STD_PARALLEL_2:
				iss >> _b2;
				_b2 /= rho;
				_eflags |= STD_PARALLEL_2;
				break;

			case REFERENCE_POINT:
				iss >> _bb >> _lb >> _hb;
				_bb /= rho;
				_lb /= rho;
				_eflags |= REFERENCE_POINT;
				break;

			case START_DATA:
				hdr = false;
				break;

			case END_DATA:
				//hdr = false;
				break;

			case UNKNOWN:
			default:
				cout << "ERROR: Keyword unknown: "
					<< keyword << endl;
				return false;
				break;
			}
		}

	//cout << line << endl;
	return true;
}


// write header

void GCLIO::write_header(ostream& os) const
{
	UnitAngle uangle;
	double rho = uangle.rho(_acode);

	//if (is_flag(VERSION))
	//	os << "version " << _ell_name << endl;

	if (is_flag(ELLIPSOID))
		os << "ellipsoid " << _ell_name << endl;

	if (is_flag(MAP_PROJECTION))
		os << "map_projection " << _map_name << endl;

	if (is_flag(UNIT_ANGLE))
		os << "unit_angle " << uangle.symbol(_acode) << endl;

	Form fix4(4); fix4.fixed().width(14);
	Form fix9(9); fix9.fixed().width(14);

	if (is_flag(ORIGIN_LATITUDE))
		os << "origin_latitude " << fix9(_b0 * rho) << endl;

	if (is_flag(ORIGIN_LONGITUDE))
		os << "origin_longitude " << fix9(_l0 * rho) << endl;

	if (is_flag(SCALE))
		os << "scale " << fix9(_m0) << endl;

	if (is_flag(OFFSET_X))
		os << "offset_x " << fix4(_x0) << endl;

	if (is_flag(OFFSET_Y))
		os << "offset_y " << fix4(_y0) << endl;

	if (is_flag(STD_PARALLEL_1))
		os << "std_parallel_1 " << fix9(_b1 * rho) << endl;

	if (is_flag(STD_PARALLEL_2))
		os << "std_parallel_2 " << fix9(_b2 * rho) << endl;

	if (is_flag(REFERENCE_POINT))
		os << "reference_point "
			<< " " << fix9(_bb * rho)
			<< " " << fix9(_lb * rho)
			<< " " << fix4(_hb) << endl;
}


// print header

void GCLIO::print_header(ostream& os) const
{
	UnitAngle uangle;
	double rho = uangle.rho(_acode);

	if (is_flag(VERSION))
		os << "Version                  :  " << _major 
			<< "." << _minor << "." << _patch << endl;

	if (is_flag(ELLIPSOID))
		os << "Ellipsoid name           :  " << _ell_name << endl;

	if (is_flag(MAP_PROJECTION))
		os << "Map projection name      :  " << _map_name << endl;

	if (is_flag(UNIT_ANGLE))
		os << "Angular unit             :  " << uangle.symbol(_acode)
			<< endl;

	Form fix4(4); fix4.fixed().width(14);
	Form fix9(9); fix9.fixed().width(14);

	if (is_flag(ORIGIN_LATITUDE))
		os << "Latitude of origin       :  b0 = " << fix9(_b0 * rho)
			<< " " << uangle.symbol(_acode) << endl;

	if (is_flag(ORIGIN_LONGITUDE))
		os << "Longitude of origin      :  l0 = " << fix9(_l0 * rho)
			<< " " << uangle.symbol(_acode) << endl;

	if (is_flag(SCALE))
		os << "Scale factor             :  m0 = " << fix4(_m0) << endl;

	if (is_flag(OFFSET_X))
		os << "Offset in x              :  x0 = " << fix4(_x0)
			<< " m\n";

	if (is_flag(OFFSET_Y))
		os << "Offset in y              :  y0 = " << fix4(_y0)
			<< " m\n";

	if (is_flag(STD_PARALLEL_1))
		os << "1st standard parallel    :  b1 = " << fix9(_b1 * rho)
			<< " " << uangle.symbol(_acode) << endl;

	if (is_flag(STD_PARALLEL_2))
		os << "2nd standard parallel    :  b2 = " << fix9(_b2 * rho)
			<< " " << uangle.symbol(_acode) << endl;

	if (is_flag(REFERENCE_POINT))
		{
		os << "Reference point" << endl;
		os << "  latitude               :  bb = " << fix9(_bb * rho)
			<< " " << uangle.symbol(_acode) << endl;
		os << "  longitude              :  lb = " << fix9(_lb * rho)
			<< " " << uangle.symbol(_acode) << endl;
		os << "  height                 :  hb = " << fix4(_hb)
			<< " m\n";
		}
}


// create map projection object

Projection* create(const GCLIO& gcl)
{
	Projection* map = 0;

	Projection::ecode type = Projection::code(gcl._map_name.c_str());

	switch (type)
		{
		case Projection::GKK:
			map = new GKK(gcl._ell_name);
			assert(map != 0);

			map->l0(gcl._l0);
			break;

		case Projection::UTM:
			map = new UTM(gcl._ell_name);
			assert(map != 0);

			map->l0(gcl._l0);
			break;

		case Projection::LAMBERT1:
			map = new Lambert1(gcl._ell_name);
			assert(map != 0);

			map->b0(gcl._b0);
			map->l0(gcl._l0);
			map->m0(gcl._m0);
			map->x0(gcl._x0);
			map->y0(gcl._y0);
			break;

		case Projection::LAMBERT2:

			map = new Lambert2(gcl._ell_name);
			assert(map != 0);

			map->b0(gcl._b0);
			map->l0(gcl._l0);
			map->m0(gcl._m0);
			map->x0(gcl._x0);
			map->y0(gcl._y0);
			dynamic_cast<Lambert2*>(map)->std_parallel(gcl._b1, gcl._b2);
			break;

		case Projection::SLK:
		case Projection::SPK:
			map = new SPK;
			break;

		case Projection::SPKLV03:
			map = new SPKLV03;
			break;

		case Projection::SPKLV95:
			map = new SPKLV95;
			break;

		default:
			break;
		}

	return map;
}

} // namespace GCL
