/* $Id: gclio.h 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_GCLIO_INCLUDED
#define DEP_GCLIO_INCLUDED

#include <cstring>
#include <iostream>
#include "uangle.h"
#include "gkk.h"
#include "utm.h"
#include "lambert1.h"
#include "lambert2.h"
#include "spk.h"

using namespace std;
using namespace GenLib2;

namespace GCL
{

/**@brief GCL file I/O class
 *
 *  This class provides some functions for input and output
 *  of GCL-files.
 *
 *  In general the GCL file is a simple text file consiting
 *  of a header block followed by the data block.
 *  The GCL format is keyword driven.
 *
 *  Header syntacs:
 *    <tt> keyword [argument ...] </tt>
 * 
 *  Data block:
 *  The data block starts with the keyword \p start_data and
 *  ends with EOF or the keyword \p end_data.
 *
 *  Keywords, arguments and description:
 *
 *    - \p start_data starts the data block
 *    - \p end_data ends the data block
 *    - \p version \<version string\> version string
 *    - \p ellipsoid \<name\> := {ell_pars.db} ellipsoid name.
 *         See data base ell_pars.db for valid ellipsoid names.
 *    - \p map_projection \<name\> map projection name
 *    - \p unit_angle \<unit\> := {rad | deg | gon}  angular unit
 *    - \p origin_latitude \<b0\> latitude of origin
 *    - \p origin_longitude \<l0\> longitude of origin
 *    - \p scale \<m0\> scale factor
 *    - \p offset_x \<x0\> offset in x
 *    - \p offset_y \<y0\> offset in y
 *    - \p std_parallel_1 \<b1\> 1st standard parallel
 *    - \p std_parallel_2 \<b2\> 2nd standard parallel
 *    - \p reference_point \<bb\> \<lb\> \<hb\> latitude, longitude and
 *      height of reference point
 *
 *  @see Ellipsoid parameter data base: tests/ell_pars.db
 *  @author Dirk Stallmann
 */
class GCLIO
{
public:
	/// header value codes
	enum ecode
	{
		UNKNOWN = 1<<0,			//!< unknown
		VERSION = 1<<1,			//!< version
		ELLIPSOID = 1<<2,		//!< ellipsoid
		MAP_PROJECTION = 1<<3,	//!< map projection
		UNIT_ANGLE = 1<<4,		//!< angular unit
		ORIGIN_LATITUDE = 1<<5,	//!< latitude of origin
		ORIGIN_LONGITUDE = 1<<6,	//!< longitude of origin
		SCALE = 1<<7,			//!< scale
		OFFSET_X = 1<<8,		//!< offset in x
		OFFSET_Y = 1<<9,		//!< offset in y
		STD_PARALLEL_1 = 1<<10,		//!< latitude of 1st standard parallel
		STD_PARALLEL_2 = 1<<11,		//!< latitude of 2nd standard parallel
		REFERENCE_POINT = 1<<12,	//!< reference point coordinates
		START_DATA = 1<<29,		//!< start of coordinate data
		END_DATA = 1<<30,		//!< end of coordinate data
		END_OF_MAP = 1<<31		//!< end of coordinate data
	};

	/**@name constructors/destructors
	 */
	//@{
		/// default constructor
		GCLIO() :
			_major(0), _minor(1), _patch(0),
			_eflags(0),
			_ell_name(),
			_acode(UnitAngle::RADIAN),
			_map_name(),
			_b0(0.), _l0(0.),
			_m0(1.),
			_x0(0.), _y0(0.),
			_b1(0.), _b2(0.),
			_bb(0.), _lb(0.), _hb(0.) {}

		/// destructor
		virtual ~GCLIO() {}
	//@}
	
	/**@brief test header value
	 *  @param[in] flg   header value codes
	 *  @return true = header value exist, false = header value exist
	 */
	bool is_flag(ecode flg) const
	{
		return (_eflags & flg) != 0 ? true : false;
	}

	/**@brief get code
	 *  @param name keyword string
	 *  @return code header value code
	 */
	static ecode code(const char* name);
	
	/**@name read/write/print file header
	 */
	//@{
		/// read header
		bool read_header(istream& is);

		/// write header
		void write_header(ostream& os) const;

		/// print header
		void print_header(ostream& os) const;
	//@}
	
	/// get angular unit code
	UnitAngle::ecode get_acode() const
	{
		return _acode;
	}

public:
	/// major version number
	int		_major;

	/// minor version number
	int		_minor;

	/// patch version number
	int		_patch;

	/// which header information is defined
	int		_eflags;

	/// ellipoid name
	string	_ell_name;

	/// angular unit
	UnitAngle::ecode _acode;

	/// map projection name
	string	_map_name;

	/// latitude of origin
	double	_b0;

	/// longitude of origin
	double	_l0;

	/// scale factor
	double	_m0;

	/// offset in x
	double	_x0;

	/// offset in y
	double	_y0;

	/// 1st standard parallel
	double	_b1;

	/// 2nd standard parallel
	double	_b2;

	/// latitude of reference point
	double	_bb;

	/// longitude of reference point
	double	_lb;

	/// height of reference point
	double	_hb;

private:
	/// map structure
	struct map_t
	{
		/// code
		int				code;

		/// keyword
		const char*		name;
	};

	/// map
	static map_t _map[];
};


// non-member functions...

/**@brief GCLIO
 *  create map projection object.
 *  
 *  In all cases the name of the reference ellipsoid is required.
 *
 *  Additional information required/used by the specific map projection:
 *
 *  GKK, UTM:
 *    latitude of origin
 *
 *  Lambert1:
 *    latitude of origin, longitude of origin, scale factor, offset in x,
 *    offset in y
 * 
 *  Lambert2:
 *    latitude of origin, longitude of origin, scale factor, offset in x,
 *    offset in y, 1st standard parallel, 2nd standard parallel
 *
 *  SPK, SPKLV03, SPKLV95:
 *     none
 *
 *  @param[in]   gcl GCL structure
 *  @return map  projection object
 */
Projection* create(const GCLIO& gcl);

} // namespace GCL

#endif
