/* $Id: gic.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#include "gic.h"

namespace GCL
{

// get map projection name

const char* GIC::map_name() const
{
	return "Gausssche isotherm coords.";
}


// To convert ellipsoidal geographic coordinates 
// into Gausssche isotherm coordinates

void GIC::geo_to_iso(double b, double l, double& x, double& y) const
{
	double sp = sin(b);
	double cp = cos(b);
	double c2p = cp * cp;
	double t = tan(b);
	double t2 = t * t;

	double ee = _e2 / (1. - _e2);
	double eta2 = ee * c2p;
	double v2 = 1. + eta2;

	// Querkruemmungsradius
	double n = _a / sqrt(1. - _e2 * sp * sp);

	double dl = l - _l0;
	double dl2 = dl * dl;

	// Rechtwert und Hochwert
	x = _m0 * (arc(b) + 0.5 * n * t * c2p * dl2
		* (1. + (5. + 9. * eta2 - t2) / 12. * c2p * dl2));
	y = _m0 * (n * cp * dl * (1. + c2p * dl2 / 6. * (v2 - t2
		+ (5. - 18. * t2 + t2 * t2) / 20. * c2p * dl2)));
	
	x += _x0;
	y += _y0;
}


// To convert Gausssche isotherm coordinates
// into ellipsoidal geographic coordinates 

void GIC::iso_to_geo(double x, double y, double& b, double& l) const
{
	x -= _x0;
	y -= _y0;

	// latitude of meridian arc
	double phi_f = inv_arc(x / _m0);

	double spf = sin(phi_f);
	double cpf = cos(phi_f);
	double tf = tan(phi_f);
	double t2f = tf * tf;

	double ee = _e2 / (1. - _e2);
	double eta2f = ee * cpf * cpf;
	double v2f = 1. + eta2f;

	// radius of curvarture in the prime vertical (Querkruemmungsradius)
	double nf = _a / sqrt(1. - _e2 * spf * spf);

	double ynf = y / nf / _m0;
	double y2nf = ynf * ynf;

	// ellipsoidal geographic coords. (ellipsoidische geographische Koord.)
	b = phi_f - 0.5 * tf * y2nf * (v2f - (5. + 3. * t2f
		+ 6. * eta2f * (1. - t2f)) / 12. * y2nf);
	l = _l0 + ynf / cpf * (1. - y2nf / 6. * (v2f + 2. * t2f
		- (5. + t2f * (28. + 24. * t2f)) / 20. * y2nf));
}

} // namespace GCL
