/* $Id: gic.h 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#ifndef GCL_GIC_INCLUDED
#define GCL_GIC_INCLUDED

#include "projection.h"

namespace GCL
{

/**@brief Gausssche isotherm coordinates
 *
 *  Conversion routines for Gausssche isotherm coordinates.
 *
 *  References:
 *  - Heck: Rechenverfahren und Auswertemodelle der Landesvermessung,
 *    Wichmann Verlag, Karlsruhe, 1987, S.257-258, Gl. (7.75) u. (7.76).
 *
 *  @author Dirk Stallmann
 */
class GIC : public Projection
{
public:
	/**@brief constructor with ellipsoid name
	 * @param ell_name  name of the reference ellipsoid
	 * @param m0    scale factor
	 * @param x0    offset in x
	 * @param y0    offset in y
	 */
	GIC(const string& ell_name, double m0=1., double x0=0., double y0=0.) :
		Projection(ell_name)
	{
		_m0 = m0;
		_x0 = x0;
		_y0 = y0;
	}

	/// get map projection name
	virtual const char* map_name() const;

	/**@name conversion routines
	 * conversion between geodetic and map coordinates
	 */
	//@{
		/**@brief convert geodetic into map coordinates
		 * @param[in]  b,l  geodetic coordinates, latitude and longitude in radian
		 * @param[out] x,y  map coordinates in meter
		 */
		void geo_to_map(double b, double l, double& x, double& y) const
		{
			geo_to_iso(b, l, x, y);
		}

		/**@brief convert map into geodetic coordinates
		 * @param[in]  x,y  map coordinates in meter
		 * @param[out] b,l  geodetic coordinates, latitude and longitude in radian
		 */
		void map_to_geo(double x, double y, double& b, double& l) const
		{
			iso_to_geo(x, y, b, l);
		}
	//@}
	
	/**@name geodetic-Gausssche isotherm coords. conversion
	 * conversion between geodetic and Gausssche isotherm coordinates
	 */
	//@{
		/**@brief convert geodetic into Gausssche isotherm coordinates
		 * @param[in]  b,l  geodetic coordinates, latitude and longitude in radian
		 * @param[out] x,y  map coordinates in meter
		 */
		void geo_to_iso(double b, double l, double& x, double& y) const;

		/**@brief convert Gausssche isotherm into geodetic coordinates
		 * @param[in]  x,y  map coordinates in meter
		 * @param[out] b,l  geodetic coordinates, latitude and longitude in radian
		 */
		void iso_to_geo(double x, double y, double& b, double& l) const;
	//@}
};

} // namespace GCL

#endif
