/* $Id: gkk.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#include "gkk.h"
using namespace GenLib2;

namespace GCL
{

// get map projection name

const char* GKK::map_name() const
{
	return "Gauss-Krueger";
}


// zone number (Kennziffer)

int GKK::zone(double l0) const
{
	return int(l0 * rho_deg / 3. + 0.5);
}


// reference meridian

double GKK::lambda0(int kz) const
{
	return (3. * kz / rho_deg);
}


// To convert geodetic into Gauss-Krueger coordinates

void GKK::geo_to_map(double b, double l, double& r, double& h) const
{
	// Kennziffer
	int kz = zone(_l0);

	double sp = sin(b);
	double cp = cos(b);
	double c2p = cp * cp;
	double t = tan(b);
	double t2 = t * t;

	double ee = _e2 / (1. - _e2);
	double eta2 = ee * c2p;
	double v2 = 1. + eta2;

	// Querkruemmungsradius
	double N = _a / sqrt(1. - _e2 * sp * sp);

	double dl = l - _l0;
	double dl2 = dl * dl;

	// Rechtwert und Hochwert
	h = arc(b) + 0.5 * N * t * c2p * dl2
		* (1. + (5. + 9. * eta2 - t2) / 12. * c2p * dl2);
	r = N * cp * dl * (1. + c2p * dl2 / 6. * (v2 - t2
		+ (5. - 18. * t2 + t2 * t2) / 20. * c2p * dl2));
	r += (kz + 0.5) * 1e6;
}


// To convert "Gauss-Krueger" into geodetic coordinates

void GKK::map_to_geo(double r, double h, double& b, double& l) const
{
	// Kennziffer is part of the Rechtswert!!!
	int kz = int(r / 1e6);
	//double l0 = 3. * kz / rho_deg;

	double x = h;
	double y = r - (kz + 0.5) * 1e6;

	// latitude of meridian arc (Meridianbogen)
	double phi_f = inv_arc(x);

	double spf = sin(phi_f);
	double cpf = cos(phi_f);
	double tf = tan(phi_f);
	double t2f = tf * tf;

	double ee = _e2 / (1. - _e2);
	double eta2f = ee * cpf * cpf;
	double v2f = 1. + eta2f;

	// radius of curvarture in the prime vertical (Querkruemmungsradius)
	double nf = _a / sqrt(1. - _e2 * spf * spf);

	double ynf = y / nf;
	double y2nf = ynf * ynf;

	// ellipsoidal geographic coords. (ellipsoidische geographische Koord.)
	b = phi_f - 0.5 * tf * y2nf * (v2f - (5. + 3. * t2f
		+ 6. * eta2f * (1. - t2f)) / 12. * y2nf);
	l = _l0 + ynf / cpf * (1. - y2nf / 6. * (v2f + 2. * t2f
		- (5. + t2f * (28. + 24. * t2f)) / 20. * y2nf));
}

} // namespace GCL
