/* $Id: gkk.h 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#ifndef GCL_GKK_INCLUDED
#define GCL_GKK_INCLUDED

#include "gic.h"

namespace GCL
{

/**@brief Conversion routines for Gauss-Krueger coordinates
 *
 *  Gauss-Krueger coordinates are Gausssche isotherm coordinates.
 *  The reference ellipsoid is often Bessel 1841.
 *
 *  References:
 *  - Heck: Rechenverfahren und Auswertemodelle der Landesvermessung,
 *    Wichmann Verlag, Karlsruhe, 1987, S.280-283.
 *
 *  @author Dirk Stallmann
 */
class GKK : public GIC
{
public:
	/**@brief constructor with ellipsoid name
	 *  @param ell_name name of the reference ellipsoid
	 */
	GKK(const string& ell_name) : GIC(ell_name, 1., 0., 500000.) {}
	
	/// get map projection name
	const char* map_name() const;

	/// calculate zone number (Kennziffer)
	int zone(double l0) const;

	/// calculate reference meridian
	double lambda0(int kz) const;

	/**@name conversion routines
	 * Conversion between geodetic (b,l) and Gauss-Krueger (R,H) coordinates.
	 * Attention: The zone number (Kennziffer) is part of the Rechtswert.
	 */
	//@{
		/**@brief convert geodetic into map coordinates
		 * @param[in]  b,l  geodetic coordinates: latitude and longitude
		 * @param[out] r,h  map coordinates: Rechtswert (R) and Hochwert (H)
		 */
		void geo_to_map(double b, double l, double& r, double& h) const;

		/**@brief convert map into geodetic coordinates
		 * @param[in]  r,h  map coordinates: Rechtswert (R) and Hochwert (H)
		 * @param[out] b,l  geodetic coordinates: latitude and longitude
		 */
		void map_to_geo(double r, double h, double& b, double& l) const;
	//@}
};

} // namespace GCL

#endif
