/* $Id: lambert2.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#include "form.h"
#include "lambert2.h"

using namespace GenLib2;

namespace GCL
{

// get map projection name

const char* Lambert2::map_name() const
{
	return "Lambert conformal conic (2SP)";
}


// print general projection parameters

void Lambert2::print(ostream& os) const
{
	Form fix8(8); fix8.fixed().width(14);

	Projection::print(os);
	os << "1st standard parallel     :  b1   = " << fix8(_b1*rho_deg)
		<< " deg\n";
	os << "2nd standard parallel     :  b2   = " << fix8(_b2*rho_deg)
		<< " deg\n";
}


// To convert geodetic coordinates into Lambert Conformal Conic coords.

void Lambert2::geo_to_map(double b, double l, double& x, double& y) const
{
	//cout << "b=" << b << endl;
	//cout << "l=" << l << endl;

	double N1 = nradius(_b1);
	double N2 = nradius(_b2);

	//cout << "q=" << q(b) << endl;
	double q0 = q(_b0);
	//cout << "q0=" << q0 << endl;
	double q1 = q(_b1);
	//cout << "q1=" << q1 << endl;
	double q2 = q(_b2);
	//cout << "q2=" << q2 << endl;

	double n = log((N1 * cos(_b1)) / (N2 * cos(_b2))) / (q2 - q1);
	//cout << "n=" << n << endl;

	double dl = n * (l - _l0);
	double R1 = N1 * cos(_b1) / n;
	//cout << "R1=" << R1 << endl;
	//double R2 = N2 * cos(_b2) / n;
	//cout << "R2=" << R2 << endl;
	//cout << "C1=" << R1 * exp(n * q1) << endl;
	//cout << "C2=" << R2 * exp(n * q2) << endl;
	double R0 = _m0 * R1 * exp(n * (q1 - q0));
	//cout << "R0=" << R0 << endl;
	double R = R0 * exp(n * (q0 - q(b)));
	//cout << "R=" << R << endl;

	x = _x0 + R * sin(dl);
	y = _y0 + R0 - R * cos(dl);
}


// To convert Lambert Conformal Conic coords. into  geodetic coordinates 

void Lambert2::map_to_geo(double x, double y, double& b, double& l) const
{
	double N1 = nradius(_b1);
	double N2 = nradius(_b2);

	double q0 = q(_b0);
	double q1 = q(_b1);
	double q2 = q(_b2);

	double n = log((N1 * cos(_b1)) / (N2 * cos(_b2))) / (q2 - q1);

	double R1 = N1 * cos(_b1) / n;
	double R0 = _m0 * R1 * exp(n * (q1 - q0));
	double dl = atan((x - _x0) / (_y0 + R0 - y));
	double dx = x - _x0;
	double dy = y - _y0 - R0;
	double R = sqrt(dx * dx + dy * dy);
	double q = q0 - log(R / R0) / n;

	b = inv_q(q);
	l = _l0 + dl / n;
}

} // namespace GCL
