/* $Id: projection.h 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#ifndef GCL_PROJECTION_INCLUDED
#define GCL_PROJECTION_INCLUDED

#include "ellipsoid.h"

namespace GCL
{

/**@brief General map projection
 *
 * This class provide a general interface for map projection
 * conversions between (ellipsoidal) geographic or geodetic (b, l)
 * and map coordinates (x, y).
 * @author Dirk Stallmann
 */
class Projection : public Ellipsoid
{
public:
	/// map projection codes
	enum ecode
	{
		UNKNOWN,	//!< unknown projection
		GKK,		//!< Gauss-Krueger-Koordinaten
		UTM,		//!< Universal Transverse Meractor projection
		LAMBERT1,	//!< Lambert Conformal Conic Projection with one standard parallel
		LAMBERT2,	//!< Lambert Conformal Conic Projection with two standard parallels
		SLK,		//!< Schweizer Landeskoordinaten
		SPK,		//!< Schweizer Projektionskoordinaten
		SPKLV03,	//!< Schweizer Landeskoordinaten LV03
		SPKLV95,	//!< Schweizer Landeskoordinaten LV95
		END_OF_MAP	//!< end-of-map
	};

	/**@name constructors/destructors
	 */
	//@{
		/// default constructor
		Projection() : Ellipsoid() { init(); }

		/**@brief constructor with name of the reference ellipsoid
		 * @param ell_name  name of the reference ellipsoid
		 */
		Projection(const string& ell_name) : Ellipsoid(ell_name)
			{ init(); }

		/// destructor
		virtual ~Projection() {}
	//@}
	
	/// get map projection name
	virtual const char* map_name() const;

	/**@name set/get latitude of origin in radian
	 */
	//@{
		/// set latitude of origin
		void b0(double b0) { _b0 = b0; }

		/// get latitude of origin 
		double b0() const { return _b0; }
	//@}
	
	/**@name set/get longitude of origin (central meridian)
	 */
	//@{
		/// set longitude of origin in radian
		void l0(double l0) { _l0 = l0; }

		/// get longitude of origin
		double l0() const { return _l0; }
	//@}
	
	/**@name set/get scale factor
	 */
	//@{
		/// set scale factor
		void m0(double m0) { _m0 = m0; }

		/// get scale factor
		double m0() const { return _m0; }
	//@}
	
	/**@name set/get offset in x (false easting)
	 */
	//@{
		/// set offset in x in meter
		void x0(double x0) { _x0 = x0; }

		/// get offset in x
		double x0() const { return _x0; }
	//@}
	
	/**@name set/get offset in y (false northing)
	 */
	//@{
		/// set offset in y
		void y0(double y0) { _y0 = y0; }

		/// get offset in y
		double y0() const { return _y0; }
	//@}

	/**@brief print parameters
	 *  @param[out] os   output stream
	 */
	virtual void print(ostream& os) const;
	
	/**@name conversion routines
	 * conversion between geodetic and map coordinates
	 */
	//@{
		/**@brief convert geodetic into map coordinates
		 * @param[in]  b,l  geodetic coordinates, latitude and longitude in radian
		 * @param[out] x,y  map coordinates in meter
		 */
		virtual void geo_to_map(double b, double l,
			double& x, double& y) const = 0;

		/**@brief convert map into geodetic coordinates
		 * @param[in]  x,y  map coordinates in meter
		 * @param[out] b,l  geodetic coordinates, latitude and longitude in radian
		 */
		virtual void map_to_geo(double x, double y,
			double& b, double& l) const = 0;
	//@}

	/**@brief get code form keyword map.
	 *  @param[in] map_name  keyword string
	 *  @return code header value code
	 */
	static ecode code(const char* map_name);

protected:
	/// latitude of origin
	double _b0;

	/// longitude of origin (central meridian)
	double _l0;

	/// scale factor
	double _m0;

	/// offset in x (false easting)
	double _x0;

	/// offset in y (false northing)
	double _y0;

	/// map structure
	struct map_t
	{
		/// code
		int				code;

		/// keyword
		const char*		name;
	};

	/// keyword map
	static map_t _map[];

private:
	/// initilize all parameters
	void init()
	{
		_b0 = _l0 = 0.;
		_m0 = 1.;
		_x0 = _y0 = 0.;
	}
};

} // namespace GCL

#endif
