/* $Id: spk.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#include "spk.h"

namespace GCL
{

// get map projection name

const char* SPK::map_name() const
{
	return "Schweizer Projektionskoordinaten";
}


// convert geodetic coordinates into Schweizer Projektionskoord.

void SPK::geo_to_map(double phi, double lam, double& y, double& x) const
{
	// CONVERSION OF ANGLES
	phi *= rho_sec / 10000.; // given latitude
	lam *= rho_sec / 10000.; // given longitude
	double P = phi - _b0 * rho_sec * 1e-4; // latitude difference
	double L = lam - _l0 * rho_sec * 1e-4; // longitude difference

	// bad polynom evaluation, use polynom factorisation!!!
	double P2 = P * P;
	double P3 = P2 * P;
	double P4 = P2 * P2;
	double P5 = P3 * P2;
	double P6 = P3 * P3;

	double L2 = L * L;
	double L3 = L2 * L;
	double L4 = L2 * L2;
	double L5 = L3 * L2;
	double L6 = L3 * L3;

	// easting (y in slk)
	double Y_1 = 2.11428533991e-1 - 1.0939608605e-2 * P
		- 2.658213e-6 * P2 - 8.539078e-6 * P3
		- 3.450e-9 * P4 - 7.992e-9 * P5;
	double Y_3 = -4.4232717e-5 + 4.291740e-6 * P
		- 3.09883e-7 * P2 + 1.3924e-8 * P3;
	double Y_5 = 1.9784e-8 - 4.277e-9 * P;
	double Y = Y_1 * L + Y_3 * L3 + Y_5 * L5;

	y = 1e6 * Y + _y0; // east

	// northing (x in slk)
	double X_0 = 3.08770746371e-1 * P + 7.5028131e-5 * P2
		+ 1.20435227e-4 * P3 + 9.488e-9 * P4
		+ 7.0332e-8 * P5 - 1.0e-11 * P6;
	double X_2 = 3.745408911e-3 - 1.93792705e-4 * P
		+ 4.340858e-6 * P2 - 3.76174e-7 * P3 + 4.053e-9 * P4;
	double X_4 = - 7.34684e-7 + 1.44466e-7 * P - 1.1842e-8 * P2;
	double X = X_0 + X_2 * L2 + X_4 * L4 + 4.88e-10 * L6;

	x = 1e6 * X + _x0; // north
}


// convert Schweizer Projektionskoord. into geodetic coordinates

void SPK::map_to_geo(double y, double x, double& phi, double& lam) const
{
	// assume scale and offset
	double X = x / 1e6 - 2e-1; // in 1000 km
	double Y = y / 1e6 - 6e-1; // in 1000 km

	double X2 = X * X;
	double X3 = X2 * X;
	double X4 = X2 * X2;
	double X5 = X3 * X2;
	double X6 = X3 * X3;

	double Y2 = Y * Y;
	double Y3 = Y2 * Y;
	double Y4 = Y2 * Y2;
	double Y5 = Y3 * Y2;
	double Y6 = Y3 * Y3;

	// longitude
	double L_1 = 4.72973056722 + 0.7925714783 * X
		+ 0.1328129667 * X2 + 2.5502202e-2 * X3
		+ 4.817474e-3 * X4 + 9.0243e-4 * X5;
	double L_3 = - 4.42709889e-2 - 2.5502202e-2 * X
		- 9.634947e-3 * X2 - 3.00808e-3 * X3;
	double L_5 = 9.63495e-4 + 9.0243e-4 * X;
	double L_G = _l0 * rho_sec * 1e-4 + L_1 * Y + L_3 * Y3 + L_5 * Y5;

	lam = 10000. * L_G / rho_sec; 

	// latitude
	double P_0 = 3.23864877666 * X - 2.5486822e-3 * X2
		- 1.32457771e-2 * X3 + 4.8747e-5 * X4
		+ 8.1305e-5 * X5 - 6.9e-7 * X6;
	double P_2 = - 0.2713537919 - 4.50442705e-2 * X
		- 7.553194e-3 * X2 - 1.463049e-3 * X3 - 2.7606e-4 * X4;
	double P_4 = 2.442786e-3 + 1.320703e-3 * X + 4.7476e-4 * X2;
	double B = _b0 * rho_sec * 1e-4 + P_0 + P_2 * Y2 + P_4 * Y4 - 4.249e-5 * Y6;

	phi = 10000. * B / rho_sec;
}

} // namespace GCL
