/* $Id: spk.h 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#ifndef GCL_SPK_INCLUDED
#define GCL_SPK_INCLUDED

#include "projection.h"

namespace GCL
{

/**@brief Schweizer Projektionskoordinaten
 *
 *  Conversion routines for Schweizer Projektionskoordinaten
 *  (Swiss oblique Mercator projection).
 *  Reference ellipsoid: Bessel.
 *  Reference point: Bern.
 *  This projection is based upon three steps:
 *    -# conformal transformation of ellipsoid coordinates to a sphere,
 *    -# rotational transformation of the sperical system so that the
 *       specified projection origin will lie on the equator, and
 *    -# application of the Mercator projection to transform to
 *       the cartesian system.
 *
 *  For references see PDF document doc/refsysd.pdf.
 *  @author Dirk Stallmann
 */
class SPK : public Projection
{
public:
	/**@brief constructor
	 *
	 *  Constructor with default offsets for the reference system LV03
	 *  x0=200000 and y0=600000.
	 *  For LV95 use the offsets x0=1200000 and y0=2600000.
	 */
	SPK(double x0=200000., double y0=600000.) :
		Projection("Bessel")
	{
		//_b0 = ((46 * 60 + 57) * 60 + 8.66) * 1e-4; // Bern latitude
		//_l0 = ((7 * 60 + 26) * 60 + 22.50) * 1e-4; // Bern longitude
		_b0 = 46 / rho_deg + 57 / rho_min +  8.66 / rho_sec; // Bern latitude
		_l0 =  7 / rho_deg + 26 / rho_min + 22.50 / rho_sec; // Bern longitude
		_x0 = x0;
		_y0 = y0;
	}

	/// get map projection name
	virtual const char* map_name() const;

	/**@name conversion routines
	 * conversion between geodetic and map coordinates
	 */
	//@{
		/**@brief convert geodetic into map coordinates
		 * @param[in]  b,l  geodetic coordinates, latitude and longitude in radian
		 * @param[out] x,y  map coordinates in meter
		 */
		void geo_to_map(double b, double l, double& y, double& x) const;

		/**@brief convert map into geodetic coordinates
		 * @param[in]  x,y  map coordinates in meter
		 * @param[out] b,l  geodetic coordinates, latitude and longitude in radian
		 */
		void map_to_geo(double y, double x, double& b, double& l) const;
	//@}
};


/**@brief Schweizer Projektionskoordinaten LV03
 *
 *  Schweizer Projektionskoordinaten for reference frame LV03.
 *  @author Dirk Stallmann
 */
class SPKLV03 : public SPK
{
public:
	/**@brief constructor
	 *
	 * Constructor using the offsets for LV03: x0=200000 and y0=600000.
	 */
	SPKLV03() : SPK(200000., 600000.) {}
};


/**@brief Schweizer Projektionskoordinaten LV95
 *
 *  Schweizer Projektionskoordinaten for reference frame LV95.
 *  @author Dirk Stallmann
 */
class SPKLV95 : public SPK
{
public:
	/**@brief constructor
	 *
	 * Constructor using the offsets for LV95: x0=1200000 and y0=2600000.
	 */
	SPKLV95() : SPK(1200000., 2600000.) {}
};

} // namespace GCL

#endif
