/* $Id: utm.h 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#ifndef GCL_UTM_INCLUDED
#define GCL_UTM_INCLUDED

#include "gic.h"

namespace GCL
{

/**@brief UTM coordinates
 *
 *  Universal Transverse Mercator (UTM) coordinates are
 *  Gausssche isotherm coordinates.
 *
 *  References:
 *  - Heck: Rechenverfahren und Auswertemodelle der Landesvermessung,
 *    Wichmann Verlag, Karlsruhe, 1987, S.280-283.
 *
 *  @author Dirk Stallmann
 */
class UTM : public GIC
{
public:
	/**@brief constructor with ellipsoid name
	 *  @param ell_name name of the reference ellipsoid
	 */
	UTM(const string& ell_name) : GIC(ell_name, 0.9996, 0., 500000.) {}

	/// get map projection name
	const char* map_name() const;

	/// calculate zone number
	int zone(double l0) const;

	/// calculate reference meridian
	double lambda0(int nz) const;

	/**@name conversion routines
	 * Conversion between geodetic (b,l) and UTM (E,N) coordinates.
	 */
	//@{
		/**@brief convert geodetic into map coordinates
		 * @param[in]  b,l  geodetic coordinates: latitude and longitude
		 * @param[out] e,n  map coordinates: East (E) and North (N)
		 */
		void geo_to_map(double b, double l, double& e, double& n) const;

		/**@brief convert map into geodetic coordinates
		 * @param[in]  e,n  map coordinates: East (E) and North (N)
		 * @param[out] b,l  geodetic coordinates: latitude and longitude
		 */
		void map_to_geo(double e, double n, double& b, double& l) const;
	//@}
};

} // namespace GCL

#endif
