/* $Id: cart2geo.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**********************************************************************/
//
// FILE:    cart2geo.cpp
//          Convert spartial cartesian into geodetic coords.
//
//          The input and output format is in GCL.
//          See gclio.h for more information.
//
//          Coordinates
//          X, Y, Z  spartial cartesian coordinates
//          b, l, h  geodectic coordinates: latitude, logitude,
//                   ellipsoid height
//
//          Required header information (input):
//            ellipsoid <ellipsoid name> := {see ell_pars.db}
//            unit_angle <unit> := {rad | deg | gon}
//
//          X, Y, Z oordinates to transform (input):
//            start_data
//            <ID> <X> <Y> <Z>
//            ...
//            [end_data]
//
//          Transformed geodetic coordinates (output):
//            start_data
//            <ID> <b> <l> <h>
//            ...
//            [end_data]
//
//          The units of the geodetic latitude and longitude are
//          defined by unit of angles in the header. The ellipsoid
//          height always is in metre.
//
// CREATED: 24 Aug 00  ds
// CHANGES: 11 Mar 01  1. Using new header names and STL strings.
//                     2. Reading and writing header line.
//                     3. string and floating-point output using
//                        Form. (ds)
//          19 Jul 02  Using string streams. (ds)
//          17 Mar 04  Using class GCLIO for file I/O. (ds)
// REFS:
//
// (c) 2000/02 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#include <cstdlib>
#include <fstream>
using namespace std;

#include "uangle.h"
#include "form.h"
using namespace GenLib2;

#include "libgcl.h"
using namespace GCL;


const char* const program_name = "cart2geo";
const char* const program_version = "version 1.1.2 (08/2008)";


static void usage(int status=0)
{
	cerr << "Usage: " << program_name << " [options] cart_file geo_file\n"
		<< "Arguments:\n"
		<< "  cart_file    Input file with cartesian coords.\n"
		<< "  geo_file     Output file with geodetic coords.\n"
		<< "Options:\n"
		<< "  -help        Show this text.\n"
		<< "  -verbose     Gives more information.\n"
		<< "  -version     Show program version.\n";
	exit(status);
};


static void error(const char* s, const char* s2="")
{
	cerr << program_name << ": " << s << ' ' << s2 << endl;
	exit(1);
}


int main(int argc, char *argv[])
{
	char*		cart_fname = 0;	// cart filename
	char*		geo_fname = 0;	// geo filename
	int			verb = 0;

	if (argc == 1) usage(0);

	int i = 0, j = 0;
	while (argc > ++i)
		{
		if (**(argv + i) == '-')
			{
			if (strcmp(argv[i], "-help") == 0)
				usage(0);
			else if (strcmp(argv[i], "-verbose") == 0)
				verb = 1;
			else if (strcmp(argv[i], "-version") == 0)
				{
				cerr << program_name << " " << program_version
					<< " compiled " << __DATE__ << " " << __TIME__ << endl;
				exit(0);
				}
			else
				error("Illegal option:", argv[i]);
			}
		else
			{
			switch (j++)
				{
				case 0:
					cart_fname = argv[i];
					break;

				case 1:
					geo_fname = argv[i];
					break;

				default:
					error("Too many arguments given:", argv[i]);
				}
			}
		}
	
	ifstream is(cart_fname);
	if (!is) error("Cannot open input data file:", cart_fname);

	ofstream os(geo_fname);
	if (!is) error("Cannot open output data file:", geo_fname);

	GCLIO gclio;
	if (gclio.read_header(is) != true)
		error("Reading header information failed.");

	if (verb)
		{
		gclio.print_header(cout);
		cout << endl;
		}

	// rho
	UnitAngle uangle;
	double rho = uangle.rho(gclio._acode);

	Ellipsoid ell;

	if (!ell.set(gclio._ell_name))
		error("Unknown ellipsoid:", gclio._ell_name.c_str());

	if (verb)
		{
		ell.print(cout);
		cout << endl;
		}

	// write header
	gclio.write_header(os);
	os << "start_data\n";

	string	id;			// point ID string
	double	b, l, h;	// latitude, lattitude, ellipsoidic height
	double	x, y, z;	// cartesain coords.
	int		num = 0;	// point counter

	Form str12; str12.left().width(12);
	Form fix4(4); fix4.fixed().width(14);
	Form fix9(9); fix9.fixed().width(14);

	while (is >> id)
		{
		if (strcmp(id.c_str(), "end_data") == 0) break;

		is >> x >> y >> z;

		ell.cart_to_geo(x, y, z, b, l, h);

		// convert in degree
		b *= rho;
		l *= rho;

		if (verb)
			{
			//cout << form("XYZ : %12s %14.4f %14.4f %14.4f\n", id, x, y, z);
			//cout << form("BLh : %12s %14.9f %14.9f %14.4f\n", id, b, l, h);
			cout << "XYZ : " << str12(id) << " " << fix4(x) << " " << fix4(y)
				<< " " << fix4(z) << endl;
			cout << "BLh : " << str12(id) << " " << fix9(b) << " " << fix9(l)
				<< " " << fix4(h) << endl;
			}

		//os << form("%12s %14.9f %14.9f %14.4f\n", id, b, l, h);
		os << str12(id) << " " << fix9(b) << " " << fix9(l)
			<< " " << fix4(h) << endl;
		num++;
		}
	os << "end_data\n";

	cout << "Number of transformed points: " << num << endl;

	return 0;
}
