/* $Id: dec2dms.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**********************************************************************/
//
// FILE:    dec2dms.cpp
//          Convert geodetic coordinates from decimal into
//          sexagesimal format.
//
//          The input and output format is in GCL.
//          See gclio.h for more information.
//
//          Coordinates
//          b, l, h  geodectic coordinates: latitude, logitude,
//                   ellipsoid height
//
//          Required header information (input):
//
//          Input of geodetic coordinates in decimal format:
//            start_data
//            <ID> <b> <l> <h>
//            ...
//            [end_data]
//
//          Output of geodetic coordinates in degree-minute-second
//          (DMS) format:
//            start_data
//            <ID> <b> <l> <h>
//            ...
//            [end_data]
//
//          Units of angles and coordinates (input and output):
//           - geodetic coords.: latitude, longitude defined by
//             unit of angles degree in the input file.
//           - ellipsoid height always in meter.
//          The ellipsoid height is not effected and ony copied.
//          The unit is always in meter.
//
// CREATED: 30 Mar 04  (ds)
// CHANGES:
// REFS:
//
// (c) 2004 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#include <cstdlib>
#include <fstream>
using namespace std;

#include "dmsconv.h"
using namespace GenLib2;

#include "libgcl.h"
using namespace GCL;


const char* const program_name = "dec2dms";
const char* const program_version = "version 0.1.2 (08/2008)";


static void usage(int status=0)
{
	cerr << "Usage: " << program_name << " [options] dec_file dms_file\n"
		<< "Arguments:\n"
		<< "  geo_file     Input file with geodetic coords.\n"
		<< "  map_file     Output file with mapping coords.\n"
		<< "Options:\n"
		<< "  -help        Show this text.\n"
		<< "  -verbose     Gives more information.\n"
		<< "  -version     Show program version.\n";
	exit(status);
}


static void error(const char* s, const char* s2="")
{
	cerr << program_name << ": " << s << ' ' << s2 << endl;
	exit(1);
}


int main(int argc, char *argv[])
{
	char*		dec_fname = 0;	// input filename
	char*		dms_fname = 0;	// output filename
	int			verb = 0;

	if (argc == 1) usage(0);

	int i = 0, j = 0;
	while (argc > ++i)
		{
		if (**(argv + i) == '-')
			{
			if (strcmp(argv[i], "-help") == 0)
				usage(0);
			else if (strcmp(argv[i], "-verbose") == 0)
				verb = 1;
			else if (strcmp(argv[i], "-version") == 0)
				{
				cerr << program_name << " " << program_version
					<< " compiled " << __DATE__ << " " << __TIME__ << endl;
				exit(0);
				}
			else
				error("Illegal option:", argv[i]);
			}
		else
			{
			switch (j++)
				{
				case 0:
					dec_fname = argv[i];
					break;

				case 1:
					dms_fname = argv[i];
					break;

				default:
					error("Too many arguments given:", argv[i]);
				}
			}
		}

	ifstream is(dec_fname);
	if (!is) error("Cannot open input data file:", dec_fname);

	ofstream os(dms_fname);
	if (!is) error("Cannot open output data file:", dms_fname);

	GCLIO gclio;
	if (gclio.read_header(is) != true)
		error("Reading header information failed.");

	if (verb)
		{
		gclio.print_header(cout);
		cout << endl;
		}

	// write header
	gclio.write_header(os);
	os << "start_data\n";

	// check angular unit
	if (gclio._acode != UnitAngle::DEGREE)
		error("Invalid angular unit.");

	string	id;				// point ID string
	double	b, l;			// geodetic coords.
	double	bd, bm, bs;		// latitude in DMS form
	double	ld, lm, ls;		// longitude coords. in DMS form
	double	h;				// ellipsoidic height
	int		num = 0;		// point counter

	Form str12; str12.left().width(12);
	Form fix_d(0); fix_d.right().fixed().width(4);
	Form fix_m(0); fix_m.right().fixed().width(2);
	Form fix_s(5); fix_s.right().fixed().width(8);
	Form fix4(4); fix4.fixed().width(4);

	while (is >> id)
		{
		if (strcmp(id.c_str(), "end_data") == 0) break;

		is >> b >> l >> h;
		dec2dms(b, bd, bm, bs);
		dec2dms(l, ld, lm, ls);

		os << str12(id) << "   "
			<< fix_d(bd) << " " << fix_m(bm)
			<< " " << fix_s(bs) << "   "
			<< fix_d(ld) << " " << fix_m(lm)
			<< " " << fix_s(ls) << "   "
			<< fix4(h) << endl;
		num++;
		}
	os << "end_data\n";

	cout << "Number of transformed points: " << num << endl;

	return 0;
}

