/* $Id: dms2dec.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**********************************************************************/
//
// FILE:    dms2dec.cpp
//          Convert geodetic coordinates sexagesimal into
//          decimal format.
//
//          The input and output format is in GCL.
//          See gclio.h for more information.
//
//          Coordinates
//          b, l, h  geodectic coordinates: latitude, logitude,
//                   ellipsoid height
//
//          Required header information (input):
//
//          Input of geodetic coordinates in degree-minute-second
//          (DMS) format:
//            start_data
//            <ID> <b> <l> <h>
//            ...
//            [end_data]
//
//          Output of geodetic coordinates in decimal format:
//            start_data
//            <ID> <b> <l> <h>
//            ...
//            [end_data]
//
//          Units of angles and coordinates (input and output):
//           - geodetic coords.: latitude, longitude defined by
//             unit of angles degree in the input file.
//           - ellipsoid height always in meter.
//          The ellipsoid height is not effected and ony copied.
//          The unit is always in meter.
//
// CREATED: 30 Mar 04  (ds)
// CHANGES:
// REFS:
//
// (c) 2004 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#include <cstdlib>
#include <fstream>
using namespace std;

#include "form.h"
using namespace GenLib2;

#include "dmsconv.h"
#include "gclio.h"
using namespace GCL;


const char* const program_name = "dms2dec";
const char* const program_version = "version 0.1.2 (08/2008)";


static void usage(int status=0)
{
	cerr << "Usage: " << program_name << " [options] geo_file map_file\n"
		<< "Arguments:\n"
		<< "  in_file      Input file in DMS format.\n"
		<< "  out_file     Output file in decimal format.\n"
		<< "Options:\n"
		<< "  -help        Show this text.\n"
		<< "  -verbose     Gives more information.\n"
		<< "  -version     Show program version.\n";
	exit(status);
}


static void error(const char* s, const char* s2="")
{
	cerr << program_name << ": " << s << ' ' << s2 << endl;
	exit(1);
}


int main(int argc, char *argv[])
{
	char*	in_fname = 0;	// input filename
	char*	out_fname = 0;	// output filename
	int		verb = 0;		// verbose flag

	if (argc == 1) usage(0);

	int i = 0, j = 0;
	while (argc > ++i)
		{
		if (**(argv + i) == '-')
			{
			if (strcmp(argv[i], "-help") == 0)
				usage(0);
			else if (strcmp(argv[i], "-verbose") == 0)
				verb = 1;
			else if (strcmp(argv[i], "-version") == 0)
				{
				cerr << program_name << " " << program_version
					<< " compiled " << __DATE__ << " " << __TIME__ << endl;
				exit(0);
				}
			else
				error("Illegal option:", argv[i]);
			}
		else
			{
			switch (j++)
				{
				case 0:
					in_fname = argv[i];
					break;

				case 1:
					out_fname = argv[i];
					break;

				default:
					error("Too many arguments given:", argv[i]);
				}
			}
		}

	ifstream is(in_fname);
	if (!is) error("Cannot open input data file:", in_fname);

	ofstream os(out_fname);
	if (!is) error("Cannot open output data file:", out_fname);

	GCLIO gclio;
	if (gclio.read_header(is) != true)
		error("Reading header information failed.");

	if (verb)
		{
		gclio.print_header(cout);
		cout << endl;
		}

	// check angular unit
	if (gclio._acode != UnitAngle::DEGREE)
		error("Invalid angular unit.");

	// rho
	//UnitAngle uangle;
	//double rho = uangle.rho(gclio._acode);

	// write header
	gclio.write_header(os);
	os << "start_data\n";

	string	id;				// point ID string
	double	d, m, s;		// latitude and lattitude
	double	b, l;			// map coords.
	double	h;				// ellipsoid height
	int		num = 0;		// point counter

	Form str12; str12.left().width(12);
	Form fix4(4); fix4.fixed().width(14);
	Form fix9(9); fix9.fixed().width(14);

	while (is >> id)
		{
		if (strcmp(id.c_str(), "end_data") == 0) break;

		// latitude
		is >> d >> m >> s;
		b = dms2dec(d, m, s);

		// longitude
		is >> d >> m >> s;
		l = dms2dec(d, m, s);

		// ell. height
		is >> h;

		os << str12(id) << " " << fix9(b) << " " << fix9(l)
			<< " " << fix4(h) << endl;
		num++;
		}
	os << "end_data\n";

	cout << "Number of transformed points: " << num << endl;

	return 0;
}

