/* $Id: gcl2shp.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**********************************************************************/
//
// FILE:    gcl2shp.cpp
//          Convert GCL data into ESRI shapefiles.
// CREATED: 12 Jul 04  (ds)
// CHANGES:
// REFS:    ESRI Shapefile Technical Description, ESRI White Paper,
//          July 1998.
//
// (c) 2004 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#include <cstdlib>
#include <cstring>
#include <iostream>
#include <sstream>
using namespace std;

#include "shapefil.h"


const char* const program_name = "gcl2shp";
const char* const program_version = "version 0.1.2 (08/2008)";


static void usage(int status=0)
{
	cerr << "Usage: " << program_name << " [options]"
		<< " shp_stem < gcl_file\n"
		<< "Arguments:\n"
		<< "  shp_stem     Stem name of output shapefiles (.shp, .shx, .dbf).\n"
		<< "  gcl_file     Input file GCL file (stdin).\n"
		<< "Options:\n"
		<< "  -gv          Using grey value as z-coord.\n"
		<< "  -help        Show this text.\n"
		<< "  -verbose     Gives more information.\n";
	exit(status);
}


static void error(const char* s, const char* s2="")
{
	cerr << program_name << ": " << s << ' ' << s2 << endl;
	exit(1);
}


int main(int argc, char* argv[])
{
	char*	out_stem = 0;			// output stem
	bool	verb = 0;				// verbose flag
	int		nSHPType = SHPT_POINTZ;	// shape type
	bool	gv_flg = false;			// gray value flag
	
	if (argc == 1) usage(0);

	int i = 0, j = 0;
	while (argc > ++i)
		{
		if (**(argv + i) == '-')
			{
			if (strcmp(argv[i], "-help") == 0)
				usage(0);
			else if (strcmp(argv[i], "-gv") == 0)
				gv_flg = true;
			else if (strcmp(argv[i], "-pointm") == 0)
				nSHPType = SHPT_POINTM;
			else if (strcmp(argv[i], "-pointz") == 0)
				nSHPType = SHPT_POINTZ;
			else if (strcmp(argv[i], "-verbose") == 0)
				verb = true;
			else if (strcmp(argv[i], "-version") == 0)
				{
				cerr << program_name << " " << program_version
					<< " compiled " << __DATE__ << " " << __TIME__ << endl;
				exit(0);
				}
			else
				error("Illegal option:", argv[i]);
			}
		else
			{
			switch (j++)
				{
				case 0:
					out_stem = argv[i];
					break;

				default:
					error("Too many arguments given:", argv[i]);
				}
			}
		}

	// create shapefile
	SHPHandle hSHPHandle = SHPCreate(out_stem, nSHPType);

	if(hSHPHandle == NULL)
		error("Cannot create shape files:", out_stem);

	/*
	cout << "Shape type: ";
	switch (nSHPType)
		{
		case SHPT_POINTM:
			cout << "PointM\n";
			break;

		case SHPT_POINTZ:
			cout << "PointZ\n";
			break;

		default:
			error("Invalid shape type.");
			break;
		}
	*/

	// create attribute file
	DBFHandle hDBFHandle = DBFCreate(out_stem);
	if(hDBFHandle == NULL)
		error("Cannot create attribute file:", out_stem);

	string	line;	// line buffer
	string	id;		// point ID string
	bool	data_flg = false;

	//cout << "Start data: ";
	while (cin)
		{
		getline(cin, line);
		istringstream iss(line);

		iss >> id;
		if (strcmp(id.c_str(), "start_data") == 0)
			{
			//cout << "found\n";
			data_flg = true;
			break;
			}
		}

	if (!data_flg)
		error("No data found in input file.");

	// create attribute file, might be required for ArcMap import
	DBFAddField(hDBFHandle, "x-coord", FTDouble, 12, 3);
	DBFAddField(hDBFHandle, "y-coord", FTDouble, 12, 3);
	
	if (gv_flg)
		DBFAddField(hDBFHandle, "gv", FTDouble, 12, 3);
	else
		DBFAddField(hDBFHandle, "z-coord", FTDouble, 12, 3);
	//DBFAddField(hDBFHandle, "dummy", FTInteger, 7, 0);
	
	int			num = 0;			// point counter
	SHPObject*	psShape = NULL;
	double		x = 0., y = 0., z = 0., m = 0.;
	double		gv = 0.;

	while (cin)
		{
		getline(cin, line);
		istringstream iss(line);

		iss >> id;
		if (strcmp(id.c_str(), "end_data") == 0) break;

		iss >> x >> y >> z; // read always xyz

		if (gv_flg)
			{
			iss >> gv;	// read grey value
			z = gv;		// use grey value instead of z
			}

		psShape = SHPCreateObject(nSHPType,
			-1, 0, NULL, NULL, 1, &x, &y, &z, &m);
		SHPWriteObject(hSHPHandle, -1, psShape);

		DBFWriteDoubleAttribute(hDBFHandle, num, 0, x);
		DBFWriteDoubleAttribute(hDBFHandle, num, 1, y);
		DBFWriteDoubleAttribute(hDBFHandle, num, 2, z);
		//DBFWriteIntegerAttribute(hDBFHandle, num, 2, gv);
		
		SHPDestroyObject(psShape);
		num++;
		}

	SHPClose(hSHPHandle);
	DBFClose(hDBFHandle);

	cout << "Number of converted points: " << num << endl;

    return 0;
}
