/* $Id: loc2geoc.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**********************************************************************/
//
// FILE:    loc2geoc.cpp
//          Convert local tangential coordinates into
//          geocentric cartesian coordinates.
//
//          The input and output format is in GCL.
//          See gclio.h for more information.
//
//          Coordinates
//          X, Y, Z  geocentric cartesian coordinates
//          x, y, z  local tangential coordinates
//
//          The local tangential system is defined by
//          the coordinates of the reference point (RF).
//
//          Required header information (input):
//            ellipsoid <ellipsoid name> := {see ell_pars.db}
//            unit_angle <unit> := {rad | deg | gon}
//            reference_point <bb> <lb> <hb>
//
//          x, y, z coordinates to transform (input):
//            start_data
//            <ID> <x> <y> <x>
//            ...
//            [end_data]
//
//          The units of local cartesian coordinates are in metre.
//
//          Transformed coordinates (output):
//            start_data
//            <ID> <X> <Y> <Z>
//            ...
//            [end_data]
//
//          The units of geocentric cartesian coordinates are in metre.
//
// CREATED: 01 May 03  ds
// CHANGES: 17 Mar 04  Using class GCLIO for file I/O. (ds)
//          21 Jul 04  Using new method loc_to_geo. (ds)
// REFS:
//
// (c) 2004 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#include <cstdlib>
#include <fstream>
using namespace std;

#include "uangle.h"
#include "form.h"
using namespace GenLib2;

#include "libgcl.h"
using namespace GCL;

const char* const program_name = "loc2geoc";
const char* const program_version = "version 1.1.2 (08/2008)";


static void usage(int status=0)
{
	cerr << "Usage: " << program_name << " [options] geoc_file loc_file\n"
		<< "Arguments:\n"
		<< "  geoc_file    Input file with geocentric cartesian coords.\n"
		<< "  loc_file     Output file with local tangential coords.\n"
		<< "Options:\n"
		<< "  -help        Show this text.\n"
		<< "  -verbose     Gives more information.\n"
		<< "  -version     Show program version.\n";
	exit(status);
};


static void error(const char* s, const char* s2="")
{
	cerr << program_name << ": " << s << ' ' << s2 << endl;
	exit(1);
}


int main(int argc, char *argv[])
{
	char*		geoc_fname = 0;	// geoc filename
	char*		loc_fname = 0;	// loc filename
	int			verb = 0;

	if (argc == 1) usage(0);

	int i = 0, j = 0;
	while (argc > ++i)
		{
		if (**(argv + i) == '-')
			{
			if (strcmp(argv[i], "-help") == 0)
				usage(0);
			else if (strcmp(argv[i], "-verbose") == 0)
				verb = 1;
			else if (strcmp(argv[i], "-version") == 0)
				{
				cerr << program_name << " " << program_version
					<< " compiled " << __DATE__ << " " << __TIME__ << endl;
				exit(0);
				}
			else
				error("Illegal option:", argv[i]);
			}
		else
			{
			switch (j++)
				{
				case 0:
					geoc_fname = argv[i];
					break;

				case 1:
					loc_fname = argv[i];
					break;

				default:
					error("Too many arguments given:", argv[i]);
				}
			}
		}
	
	ifstream is(geoc_fname);
	if (!is) error("Cannot open input data file:", geoc_fname);

	ofstream os(loc_fname);
	if (!is) error("Cannot open output data file:", loc_fname);

	GCLIO gclio;
	if (gclio.read_header(is) != true)
		error("Reading header information failed.");

	if (verb)
		{
		gclio.print_header(cout);
		cout << endl;
		}

	// rho
	UnitAngle uangle;
	double rho = uangle.rho(gclio._acode);

	ConvGeocLoc trf;

	if (!trf.set(gclio._ell_name))
		error("Unknown ellipsoid:", gclio._ell_name.c_str());

	trf.set_reference_point(gclio._bb, gclio._lb, gclio._hb);

	if (verb)
		{
		trf.print(cout);
		cout << endl;
		}

	// write header
	gclio.write_header(os);
	os << "start_data\n";

	string	id;			// point ID string
	double	b, l, h;	// geodetic coords.
	double	x, y, z;	// local coords.
	int		num = 0;	// point counter

	Form str12; str12.left().width(12);
	Form fix4(4); fix4.fixed().width(14);
	Form fix9(9); fix9.fixed().width(14);

	string line;
	string comment;
	while (is)
		{
		getline(is, line);
		istringstream iss(line);

		iss >> id;
		if (strcmp(id.c_str(), "end_data") == 0) break;

		iss >> x >> y >> z;

		trf.loc_to_geo(x, y, z, b, l, h);

		b *= rho;
		l *= rho;

		if (verb)
			{
			//cout << form("XYZ : %12s %14.4f %14.4f %14.4f\n", id, x, y, z);
			//cout << form("BLh : %12s %14.9f %14.9f %14.4f\n", id, b, l, h);
			cout << "TOP : " << str12(id) << " " << fix4(x)
				<< " " << fix4(y) << " " << fix4(z) << endl;
			cout << "GEO : " << str12(id) << " " << fix9(b)
				<< " " << fix9(l) << " " << fix4(h) << endl;

			}

		getline(iss, comment); // read comment

		//os << form("%12s %14.9f %14.9f %14.4f\n", id, b, l, h);
		os << str12(id) << " "
			<< fix9(b) << " " << fix9(l) << " " << fix4(h);

		// write comment when exist
		if (!comment.empty()) os << " " << comment;

		os << endl;

		num++;
		}
	os << "end_data\n";

	cout << "Number of transformed points: " << num << endl;

	return 0;
}
