/* $Id: map2geo.cpp 103 2008-08-07 14:08:33Z dirk $ */

/* GCL - Geodetic Computation Library
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**********************************************************************/
//
// FILE:    map2geo.cpp
//          Convert mapping into geodetic coordinates.
//
//          The input and output format is in GCL.
//          See gclio.h for more information.
//
//          Coordinates
//          b, l  geodectic coordinates: latitude, logitude
//          E, N  mapping coordinates: east value, north value
//          h     ellipsoid height
//
//          Required header information (input):
//            ellipsoid <ellipsoid name> := {see ell_pars.db}
//            unit_angle <unit> := {rad | deg | gon}
//            <map projection name> := {see projection.cpp}
//            [some projection specific parameters] := {see projection.cpp}
//
//          E, N, h mapping coordinates to transform (input):
//            start_data
//            <ID> <E> <N> <h>
//            ...
//            [end_data]
//
//          The units of the mapping coordinates and the
//          ellipsoid height are always in meter.
//
//          Transformed coordinates (output):
//            start_data
//            <ID> <b> <l> <h>
//            ...
//            [end_data]
//
//          The units of geodetic coords. are defined by
//          the unit of angles keyword in the input file
//          while the ellipsoid height is always in meter.
//          The ellipsoid height is not affected and only copied!
//
// CREATED: 22 Aug 00  (ds)
// CHANGES: 22 May 01  1. Change file header to have more general information.
//                     2. Add new projection: Lambert conformal conic. (ds)
//          11 Mar 01  1. Using new header names, STL strings, string streams.
//                     2. Reading and writing header line.
//                     3. string and floating-point output using Form. (ds)
//          19 Jul 02  Using string streams and bug fix in setting map
//                     projection parameters. (ds)
//          17 Mar 04  Using class GCLIO for file I/O and
//                     add new projection Schweizer Projektions-
//                     koordinaten. (ds)
// REFS:
//
// (c) 2000 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#include <cstdlib>
#include <fstream>
using namespace std;

#include "uangle.h"
using namespace GenLib2;

#include "libgcl.h"
using namespace GCL;


const char* const program_name = "map2geo";
const char* const program_version = "version 1.1.2 (08/2008)";


static void usage(int status=0)
{
	cerr << "Usage: " << program_name << " [options] map_file geo_file\n"
		<< "Arguments:\n"
		<< "  map_file     Input file with mapping coords.\n"
		<< "  geo_file     Output file with geodetic coords.\n"
		<< "Options:\n"
		<< "  -help        Show this text.\n"
		<< "  -verbose     Gives more information.\n"
		<< "  -version     Show program version.\n";
	exit(status);
}


static void error(const char* s, const char* s2="")
{
	cerr << program_name << ": " << s << ' ' << s2 << endl;
	exit(1);
}


int main(int argc, char *argv[])
{
	char*		map_fname = 0;	// map filename
	char*		geo_fname = 0;	// geo filename
	int			verb = 0;

	if (argc == 1) usage(0);

	int i = 0, j = 0;
	while (argc > ++i)
		{
		if (**(argv + i) == '-')
			{
			if (strcmp(argv[i], "-help") == 0)
				usage(0);
			else if (strcmp(argv[i], "-verbose") == 0)
				verb = 1;
			else if (strcmp(argv[i], "-version") == 0)
				{
				cerr << program_name << " " << program_version
					<< " compiled " << __DATE__ << " " << __TIME__ << endl;
				exit(0);
				}
			else
				error("Illegal option:", argv[i]);
			}
		else
			{
			switch (j++)
				{
				case 0:
					map_fname = argv[i];
					break;

				case 1:
					geo_fname = argv[i];
					break;

				default:
					error("Too many arguments given:", argv[i]);
				}
			}
		}

	ifstream is(map_fname);
	if (!is) error("Cannot open input data file:", map_fname);

	ofstream os(geo_fname);
	if (!is) error("Cannot open output data file:", geo_fname);

	GCLIO gclio;
	if (gclio.read_header(is) != true)
		error("Reading header information failed.");

	if (verb)
		{
		gclio.print_header(cout);
		cout << endl;
		}

	// rho
	UnitAngle uangle;
	double rho = uangle.rho(gclio._acode);

	Projection *map = create(gclio);
	if (map == 0)
		error("Unknown map projection:", gclio._map_name.c_str());

	//if (!map->is_valid())
	//	error("Invalid reference ellipsoid:", gclio._ell_name.c_str());

	if (verb)
		{
		map->print(cout);
		cout << endl;
		map->Ellipsoid::print(cout);
		cout << endl;
		}

	// write header
	gclio.write_header(os);
	os << "start_data\n";

	string	id;					// point ID string
	//Angle	ba(uangle, hms_flg);	// latitude
	//Angle	la(uangle, hms_flg);	// longitude
	double	b, l;				// latitude and lattitude
	double	e, n;				// map coords.
	double	h;					// ellipsoidic height
	int		num = 0;			// point counter

	Form str12; str12.left().width(12);
	Form fix4(4); fix4.fixed().width(14);
	Form fix9(9); fix9.fixed().width(14);

	while (is >> id)
		{
		if (strcmp(id.c_str(), "end_data") == 0) break;

		is >> e >> n >> h;

		// transformation into geodetic coords.
		map->map_to_geo(e, n, b, l);

		/*
		if (verb)
			{
			cout << form("ENh : %14.4f %14.4f %14.4f [%.1f]\n",
				e, n, h, l0);
			cout << form("BLh : %14.9f %14.9f %14.4f [%d]\n",
				b, l, h, nz);
			}
		*/

		//ba.rad(b);
		//la.rad(l);
		b *= rho;
		l *= rho;
		os << str12(id) << " " << fix9(b) << " " << fix9(l)
			<< " " << fix4(h) << endl;
		num++;
		}
	os << "end_data\n";

	cout << "Number of transformed points: " << num << endl;

	return 0;
}

